@@ -47,17 +47,7 @@
 
 #include "rt-sched.h"
 #include "rt-utils.h"
-
-#include "error.h"
-
-/* conversions */
-#define USEC_PER_SEC 	1000000
-#define NSEC_PER_SEC 	1000000000
-#define USEC_TO_NSEC(u) ((u) * 1000)
-#define USEC_TO_SEC(u) 	((u) / USEC_PER_SEC)
-#define NSEC_TO_USEC(n) ((n) / 1000)
-#define SEC_TO_NSEC(s) 	((s) * NSEC_PER_SEC)
-#define SEC_TO_USEC(s) 	((s) * USEC_PER_SEC)
+#include "rt-error.h"
 
 /* test timeout */
 #define TIMEOUT 2
@@ -109,6 +99,9 @@ int debugging = 0;
 
 int quiet = 0;	/* turn off all prints, default = 0 (off) */
 
+/* filename for JSON output */
+char jsonfile[MAX_PATH];
+
 /* prompt to start test */
 int prompt = 0;
 
@@ -130,24 +123,6 @@ int uniprocessor = 0;
 /* lock all memory */
 int lockall = 0;
 
-/* command line options */
-struct option options[] = {
-	{"duration", required_argument, NULL, 'D'},
-	{"verbose", no_argument, NULL, 'v'},
-	{"quiet", no_argument, NULL, 'q'},
-	{"groups", required_argument, NULL, 'g'},
-	{"inversions", required_argument, NULL, 'i'},
-	{"rr", no_argument, NULL, 'r'},
-	{"sched", required_argument, NULL, 's'},
-	{"uniprocessor", no_argument, NULL, 'u'},
-	{"prompt", no_argument, NULL, 'p'},
-	{"debug", no_argument, NULL, 'd'},
-	{"version", no_argument, NULL, 'V'},
-	{"mlockall", no_argument, NULL, 'm'},
-	{"help", no_argument, NULL, 'h'},
-	{NULL, 0, NULL, 0},
-};
-
 #define NUM_TEST_THREADS 3
 #define NUM_ADMIN_THREADS 1
 
@@ -228,7 +203,7 @@ int setup_thread_attr(pthread_attr_t * attr, struct sched_attr * sa,
 		      cpu_set_t * mask);
 int set_cpu_affinity(cpu_set_t * test_mask, cpu_set_t * admin_mask);
 void process_command_line(int argc, char **argv);
-void usage(void);
+void usage(int error);
 int block_signals(void);
 int allow_sigterm(void);
 void set_shutdown_flag(void);
@@ -237,9 +212,10 @@ int create_group(struct group_parameters *group);
 unsigned long total_inversions(void);
 void banner(void);
 void summary(void);
+void write_stats(FILE *f, void *data);
 void wait_for_termination(void);
-int barrier_init(pthread_barrier_t * b, const pthread_barrierattr_t * attr,
-		 unsigned count, const char *name);
+int barrier_init(pthread_barrier_t *b, const pthread_barrierattr_t *attr,
+		 unsigned int count, const char *name);
 void setup_sched_attr(struct sched_attr *attr, int policy, int prio);
 void setup_sched_config(int policy);
 
@@ -263,6 +239,7 @@ int main(int argc, char **argv)
 
 
 	/* process command line arguments */
+	rt_init(argc, argv);
 	process_command_line(argc, argv);
 
 	/* set default sched attributes */
@@ -327,6 +304,7 @@ int main(int argc, char **argv)
 	}
 	/* report */
 	banner();
+	rt_test_start();
 	start = time(NULL);
 
 	/* turn loose the threads */
@@ -363,6 +341,10 @@ int main(int argc, char **argv)
 		kill(0, SIGTERM);
 	finish = time(NULL);
 	summary();
+
+	if (strlen(jsonfile) != 0)
+		rt_write_json(jsonfile, retval, write_stats, NULL);
+
 	if (lockall)
 		munlockall();
 	exit(retval);
@@ -519,14 +501,6 @@ int pending_interrupt(void)
 	return interrupted = sigismember(&pending, SIGINT);
 }
 
-static inline void tsnorm(struct timespec *ts)
-{
-	while (ts->tv_nsec >= NSEC_PER_SEC) {
-		ts->tv_nsec -= NSEC_PER_SEC;
-		ts->tv_sec++;
-	}
-}
-
 /*
  * this routine serves two purposes:
  *   1. report progress
@@ -1006,31 +980,36 @@ void *high_priority(void *arg)
 	return NULL;
 }
 
-void usage(void)
+void usage(int error)
 {
-	printf("usage: pi_stress <options>\n");
-	printf("    options:\n");
-	printf("\t--verbose\t- lots of output\n");
-	printf("\t--quiet\t\t- suppress running output\n");
-	printf
-	    ("\t--duration=<n>\t- length of the test run in seconds [infinite]\n");
-	printf("\t\t\t  Append 'm', 'h', or 'd' to specify minutes, hours or days.\n");
-	printf("\t--groups=<n>\t- set the number of inversion groups [%d]\n",
-	       ngroups);
-	printf
-	    ("\t--inversions=<n>- number of inversions per group [infinite]\n");
-	printf("\t--rr\t\t- use SCHED_RR for test threads [SCHED_FIFO]\n");
-	printf("\t--sched\t\t- scheduling options per thread type:\n");
-	printf("\t\tid=[high|med|low]\t\t\t- select thread\n");
-	printf("\t\t,policy=[fifo,rr],priority=<n>\t\t- SCHED_FIFO or SCHED_RR\n");
-	printf("\t\t,policy=deadline,runtime=<n>,deadline=<n>,period=<n>\t- SCHED_DEADLINE\n");
-	printf("\t--prompt\t- prompt before starting the test\n");
-	printf
-	    ("\t--uniprocessor\t- force all threads to run on one processor\n");
-	printf("\t--mlockall\t- lock current and future memory\n");
-	printf("\t--debug\t\t- turn on debug prints\n");
-	printf("\t--version\t- print version number on output\n");
-	printf("\t--help\t\t- print this message\n");
+	printf("pi_stress V %1.2f\n", VERSION);
+	printf("Usage:\n"
+	       "pi_stress <options>\n\n"
+	       "-d       --debug           turn on debug prints\n"
+	       "-D TIME  --duration=TIME   length of test run in seconds (default is infinite)\n"
+	       "                           Append 'm', 'h', or 'd'\n"
+	       "                           to specify minutes, hours or days.\n"
+	       "-g N     --groups=N        set the number of inversion groups\n"
+	       "-h       --help            print this message\n"
+	       "-i INV   --inversions=INV  number of inversions per group (default is infinite)\n"
+	       "         --json=FILENAME   write final results into FILENAME, JSON formatted\n"
+	       "-m       --mlockall        lock current and future memory\n"
+	       "-p       --prompt          prompt before starting the test\n"
+	       "-q       --quiet           suppress running output\n"
+	       "-r       --rr              use SCHED_RR for test threads [SCHED_FIFO]\n"
+	       "-s OPTS  --sched OPTS      scheduling options per thread type:\n"
+	       "   tid=[high|med|low]      select thread\n"
+	       "   ,policy=[fifo,rr]       scheduling class [SCHED_FIFO, SCHED_RR]\n"
+	       "     ,priority=N           scheduling priority\n"
+	       "   ,policy=[deadline]      scheduling class [DEADLINE]\n"
+               "     ,runtime=N\n"
+	       "     ,deadline=N\n"
+	       "     ,period=N\n"
+	       "-u       --uniprocessor    force all threads to run on one processor\n"
+	       "-v       --verbose         lots of output\n"
+	       "-V       --version         print version number on output\n"
+	       );
+	exit(error);
 }
 
 /* block all signals (called from main) */
@@ -1306,54 +1285,105 @@ int process_sched_line(const char *arg)
 	return retval;
 }
 
+enum option_values {
+	OPT_DEBUG=1, OPT_DURATION, OPT_GROUPS, OPT_HELP, OPT_INVERSIONS,
+	OPT_JSON, OPT_MLOCKALL, OPT_PROMPT, OPT_QUIET, OPT_RR, OPT_SCHED,
+	OPT_UNIPROCESSOR, OPT_VERBOSE, OPT_VERSION,
+};
+
 void process_command_line(int argc, char **argv)
 {
-	int opt;
-	while ((opt = getopt_long(argc, argv, "+hD:vqi:g:rs:pdVum", options, NULL)) != -1) {
-		switch (opt) {
-		case 'h':
-			usage();
-			exit(0);
+	for (;;) {
+		struct option options[] = {
+			{"debug",		no_argument,		NULL, OPT_DEBUG},
+			{"duration",		required_argument,	NULL, OPT_DURATION},
+			{"groups",		required_argument,	NULL, OPT_GROUPS},
+			{"help",		no_argument,		NULL, OPT_HELP},
+			{"inversions",		required_argument,	NULL, OPT_INVERSIONS},
+			{"json",		required_argument,	NULL, OPT_JSON},
+			{"mlockall",		no_argument,		NULL, OPT_MLOCKALL},
+			{"prompt",		no_argument,		NULL, OPT_PROMPT},
+			{"quiet",		no_argument,		NULL, OPT_QUIET},
+			{"rr",			no_argument,		NULL, OPT_RR},
+			{"sched",		required_argument,	NULL, OPT_SCHED},
+			{"uniprocessor",	no_argument,		NULL, OPT_UNIPROCESSOR},
+			{"verbose",		no_argument,		NULL, OPT_VERBOSE},
+			{"version",		no_argument,		NULL, OPT_VERSION},
+			{NULL, 0, NULL, 0},
+		};
+
+		int c = getopt_long(argc, argv, "+hD:vqi:g:rs:pdVum", options, NULL);
+		if (c == -1)
+			break;
+		switch (c) {
+		case OPT_DEBUG:
+		case 'd':
+			debugging = 1;
+			break;
+		case OPT_DURATION:
 		case 'D':
 			duration = parse_time_string(optarg);
 			break;
-		case 'v':
-			verbose = 1;
-			quiet = 0;
+		case OPT_GROUPS:
+		case 'g':
+			ngroups = strtol(optarg, NULL, 10);
+			if (ngroups > num_processors) {
+					pi_error("the number of groups cannot exceed "
+					 "the number of online processors (%ld)\n",
+					 num_processors);
+				exit(-1);
+			}
+			pi_info("number of groups set to %d\n", ngroups);
 			break;
-		case 'q':
-			verbose = 0;
-			quiet = 1;
+		case OPT_HELP:
+		case 'h':
+			usage(0);
 			break;
+		case OPT_INVERSIONS:
 		case 'i':
 			inversions = strtol(optarg, NULL, 10);
 			pi_info("doing %d inversion per group\n", inversions);
 			break;
-		case 'g':
-			ngroups = strtol(optarg, NULL, 10);
-			pi_info("number of groups set to %d\n", ngroups);
+		case OPT_JSON:
+			strncpy(jsonfile, optarg, strnlen(optarg, MAX_PATH-1));
 			break;
+		case OPT_MLOCKALL:
+		case 'm':
+			lockall = 1;
+			break;
+		case OPT_PROMPT:
+		case 'p':
+			prompt = 1;
+			break;
+		case OPT_QUIET:
+		case 'q':
+			verbose = 0;
+			quiet = 1;
+			break;
+		case OPT_RR:
 		case 'r':
 			policy = SCHED_RR;
 			break;
+		case OPT_SCHED:
 		case 's':
 			if (process_sched_line(optarg))
 				pi_error("ignoring invalid options '%s'\n", optarg);
 			break;
-		case 'p':
-			prompt = 1;
-			break;
-		case 'd':
-			debugging = 1;
-			break;
-		case 'V':
-			printf("pi_stress v%1.2f ", VERSION);
-			exit(0);
+		case OPT_UNIPROCESSOR:
 		case 'u':
 			uniprocessor = 1;
 			break;
-		case 'm':
-			lockall = 1;
+		case OPT_VERBOSE:
+		case 'v':
+			verbose = 1;
+			quiet = 0;
+			break;
+		case OPT_VERSION:
+		case 'V':
+			printf("pi_stress v%1.2f ", VERSION);
+			exit(0);
+		default:
+			usage(1);
 			break;
 		}
 	}
@@ -1424,9 +1454,14 @@ void summary(void)
 	       t->tm_yday, t->tm_hour, t->tm_min, t->tm_sec);
 }
 
+void write_stats(FILE *f, void *data)
+{
+	fprintf(f, "  \"inversion\": %lu\n", total_inversions());
+}
+
 int
-barrier_init(pthread_barrier_t * b, const pthread_barrierattr_t * attr,
-	     unsigned count, const char *name)
+barrier_init(pthread_barrier_t *b, const pthread_barrierattr_t *attr,
+	     unsigned int count, const char *name)
 {
 	int status;
 
