@@ -30,9 +30,9 @@
 #include <linux/unistd.h>
 #include <linux/magic.h>
 
-#include <rt-utils.h>
-#include <rt-sched.h>
-#include <error.h>
+#include "rt-utils.h"
+#include "rt-sched.h"
+#include "rt-error.h"
 
 #define _STR(x) #x
 #define STR(x) _STR(x)
@@ -47,23 +47,6 @@ typedef unsigned long long u64;
 typedef unsigned int u32;
 typedef int s32;
 
-/* Struct to transfer parameters to the thread */
-struct thread_param {
-	u64 runtime_us;
-	u64 deadline_us;
-
-	int mode;
-	int timermode;
-	int signal;
-	int clock;
-	unsigned long max_cycles;
-	struct thread_stat *stats;
-	unsigned long interval;
-	int cpu;
-	int node;
-	int tnum;
-};
-
 /* Struct for statistics */
 struct thread_stat {
 	unsigned long cycles;
@@ -102,13 +85,11 @@ static pthread_barrier_t barrier;
 
 static int cpu_count;
 static int all_cpus;
-
 static int nr_threads;
 static int use_nsecs;
-
 static int mark_fd;
-
 static int quiet;
+static char jsonfile[MAX_PATH];
 
 static int find_mount(const char *mount, char *debugfs)
 {
@@ -578,13 +559,11 @@ static void destroy_cpuset(const char *name, int print)
 	sleep(1);
 	ret = rmdir(path);
 	if (ret < 0) {
-		if (retry++ < 5)
-			goto again;
-		err_msg_n(errno, "Failed to remove %s\n", path);
 		if (retry++ < 5) {
 			err_msg("Trying again\n");
 			goto again;
 		}
+		err_msg_n(errno, "Failed to remove %s\n", path);
 	}
 }
 
@@ -620,16 +599,17 @@ static void usage(int error)
 	       "                           tasks on. An empty CPUSET runs on all CPUs a deadline\n"
 	       "                           task.\n"
 	       "                           on CPU 4, and thread #5 on CPU 5.\n"
-	       "-D TIME     --duration     Specify a length for the test run.\n"
+	       "-D TIME  --duration        Specify a length for the test run.\n"
 	       "                           Append 'm', 'h', or 'd' to specify minutes, hours or\n"
 	       "                           days\n"
-	       "-h          --help         Show this help menu.\n"
-	       "-i INTV     --interval     The shortest deadline for the tasks in us\n"
+	       "-h       --help            Show this help menu.\n"
+	       "-i INTV  --interval        The shortest deadline for the tasks in us\n"
 	       "                           (default 1000us).\n"
-	       "-s STEP     --step         The amount to increase the deadline for each task in us\n"
+	       "         --json=FILENAME   write final results into FILENAME, JSON formatted\n"
+	       "-s STEP  --step            The amount to increase the deadline for each task in us\n"
 	       "                           (default 500us).\n"
-	       "-t NUM      --threads      The number of threads to run as deadline (default 1).\n"
-	       "-q          --quiet        print a summary only on exit\n"
+	       "-t NUM   --threads         The number of threads to run as deadline (default 1).\n"
+	       "-q       --quiet           print a summary only on exit\n"
 	       );
 	exit(error);
 }
@@ -983,6 +963,32 @@ static void loop(struct sched_data *sched_data, int nr_threads)
 	}
 }
 
+static void write_stats(FILE *f, void *data)
+{
+	struct sched_data *sd = data;
+	struct thread_stat *s;
+	unsigned int i;
+
+	fprintf(f, "  \"num_threads\": %d,\n", nr_threads);
+	fprintf(f, "  \"resolution_in_ns\": %u,\n", use_nsecs);
+	fprintf(f, "  \"thread\": {\n");
+	for (i = 0; i < nr_threads; i++) {
+		s = &sd[i].stat;
+		fprintf(f, "    \"%u\": {\n", i);
+		fprintf(f, "	 \"cycles\": %ld,\n", s->cycles);
+		fprintf(f, "	 \"min\": %ld,\n", s->min);
+		fprintf(f, "	 \"max\": %ld,\n", s->max);
+		fprintf(f, "	 \"avg\": %.2f\n", s->avg/s->cycles);
+		fprintf(f, "    }%s\n", i == nr_threads - 1 ? "" : ",");
+	}
+	fprintf(f, "  }\n");
+}
+
+enum options_valud {
+	OPT_AFFINITY=1, OPT_DURATION, OPT_HELP, OPT_INTERVAL,
+	OPT_JSON, OPT_STEP, OPT_THREADS, OPT_QUIET
+};
+
 int main(int argc, char **argv)
 {
 	struct sched_data *sched_data;
@@ -1003,24 +1009,29 @@ int main(int argc, char **argv)
 	int i;
 	int c;
 
+	rt_init(argc, argv);
+
 	cpu_count = sysconf(_SC_NPROCESSORS_CONF);
 	if (cpu_count < 1)
 		err_quit("Can not calculate number of CPUS\n");
 
 	for (;;) {
 		static struct option options[] = {
-			{ "affinity",	optional_argument,	NULL,	'a' },
-			{ "duration",	required_argument,	NULL,	'D' },
-			{ "help",	no_argument,		NULL,	'h' },
-			{ "interval",	required_argument,	NULL,	'i' },
-			{ "threads",	required_argument,	NULL,	't' },
-			{ "quiet",	no_argument,		NULL,	'q' },
+			{ "affinity",	optional_argument,	NULL,	OPT_AFFINITY },
+			{ "duration",	required_argument,	NULL,	OPT_DURATION },
+			{ "help",	no_argument,		NULL,	OPT_HELP },
+			{ "interval",	required_argument,	NULL,	OPT_INTERVAL },
+			{ "json",	required_argument,	NULL,	OPT_JSON },
+			{ "step",	required_argument,	NULL,	OPT_STEP },
+			{ "threads",	required_argument,	NULL,	OPT_THREADS },
+			{ "quiet",	no_argument,		NULL,	OPT_QUIET },
 			{ NULL,		0,			NULL,	0   },
 		};
-		c = getopt_long(argc, argv, "a::c:D:hi:t:q", options, NULL);
+		c = getopt_long(argc, argv, "a::c:D:hi:s:t:q", options, NULL);
 		if (c == -1)
 			break;
 		switch (c) {
+		case OPT_AFFINITY:
 		case 'a':
 		case 'c':
 			if (!nr_threads)
@@ -1032,21 +1043,30 @@ int main(int argc, char **argv)
 			else
 				all_cpus = 1;
 			break;
+		case OPT_INTERVAL:
 		case 'i':
 			interval = atoi(optarg);
 			break;
+		case OPT_JSON:
+			strncpy(jsonfile, optarg, strnlen(optarg, MAX_PATH-1));
+			break;
+		case OPT_STEP:
 		case 's':
 			step = atoi(optarg);
 			break;
+		case OPT_THREADS:
 		case 't':
 			nr_threads = atoi(optarg);
 			break;
+		case OPT_DURATION:
 		case 'D':
 			duration = parse_time_string(optarg);
 			break;
+		case OPT_QUIET:
 		case 'q':
 			quiet = 1;
 			break;
+		case OPT_HELP:
 		case 'h':
 			usage(0);
 			break;
@@ -1193,6 +1213,7 @@ int main(int argc, char **argv)
 	if (duration)
 		alarm(duration);
 
+	rt_test_start();
 	loop(sched_data, nr_threads);
 
 	for (i = 0; i < nr_threads; i++) {
@@ -1206,6 +1227,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (strlen(jsonfile) != 0)
+		rt_write_json(jsonfile, 0, write_stats, sched_data);
+
 	if (setcpu_buf)
 		free(setcpu_buf);
 	free(thread);
